@@ -1,4 +1,4 @@
-#' List recognized features in OSM 
+#' List recognized features in OSM
 #'
 #' @return character vector of all known features
 #'
@@ -14,18 +14,17 @@
 available_features <- function() {
 
     url_ftrs <- "https://wiki.openstreetmap.org/wiki/Map_Features"
-    if (curl::has_internet ())
-    {
+
+    if (curl::has_internet ()) {
         pg <- xml2::read_html (httr::GET (url_ftrs))
         keys <- xml2::xml_attr (rvest::html_nodes (pg, "a[href^='/wiki/Key']"), #nolint
-                                "href") %>% 
-            strsplit ("/wiki/Key:") %>% 
+                                "href") %>%
+            strsplit ("/wiki/Key:") %>%
             unlist ()
         keys [keys != ""] %>%
-            sort () %>% 
+            sort () %>%
             unique ()
-    } else
-    {
+    } else {
         message ("No internet connection")
     }
 }
@@ -49,37 +48,43 @@ available_tags <- function(feature) {
     url_ftrs <- "https://wiki.openstreetmap.org/wiki/Map_Features"
 
     ret <- NULL
-    if (curl::has_internet ())
-    {
+    if (curl::has_internet ()) {
+
         if (missing (feature))
             stop ("Please specify feature")
 
         pg <- xml2::read_html (httr::GET (url_ftrs))
+
         taglists <- rvest::html_nodes (pg, "div[class='taglist']") %>%
             rvest::html_attr ("data-taginfo-taglist-tags")
-        taglists <- lapply (taglists, function (i)
-        {
+
+        taglists <- lapply (taglists, function (i) {
+
             temp <- strsplit (i, "=") [[1]]
             res <- NULL
-            if (length (temp) == 2)
-            {
+
+            if (length (temp) == 2) {
                 res <- strsplit (temp [2], ",")
                 names (res) <- temp [1]
             }
+
             return (res)
         })
+
         taglists [vapply (taglists, is.null, logical (1))] <- NULL
         keys <- unique (unlist (lapply (taglists, names)))
-        
-        if (!(feature %in% keys))
-        {
+
+        if (!(feature %in% keys)) {
+
             # try old style tables
-            tags <- rvest::html_nodes (pg, sprintf ("a[title^='Tag:%s']", feature))
+            tags <- rvest::html_nodes (pg,
+                           sprintf ("a[title^='Tag:%s']", feature))
             tags <- vapply (strsplit (xml2::xml_attr (tags, "href"), "%3D"),
                             function (i) i [2], character (1))
             ret <- unique (sort (tags))
-        } else 
-        {
+
+        } else {
+
             taglists <- stats::setNames (do.call (mapply,
                               c (FUN = c, lapply (taglists, `[`, keys))), keys)
 
