@@ -11,8 +11,8 @@
 #' @param x An 'osmdata_sf' object returned from function of same name
 #' @return Same object, yet with no row names on geometry objects.
 #' @export
-unname_osmdata_sf <- function (x)
-{
+unname_osmdata_sf <- function (x) {
+
     requireNamespace ("sf")
 
     x <- unname_osm_points (x)
@@ -24,32 +24,39 @@ unname_osmdata_sf <- function (x)
     return (x)
 }
 
-unname_osm_points <- function (x)
-{
+unname_osm_points <- function (x) {
+
     if (nrow (x$osm_points) > 0)
         names (x$osm_points$geometry) <- NULL
     return (x)
 }
 
-unname_osm <- function (x, what = "osm_lines")
-{
+unname_osm <- function (x, what = "osm_lines") {
+
     if (is.null (x [[what]]))
         return (x)
 
     g <- lapply (x [[what]]$geometry, function (i) unname (i))
     names (g) <- NULL
-    if (what == "osm_polygons")
+
+    if (what == "osm_polygons") {
+
         g <- lapply (g, function (i) {
                          rownames (i [[1]]) <- NULL
                          return (i) })
-    else if (what == "osm_multilines")
+
+    } else if (what == "osm_multilines") {
+
         g <- lapply (g, function (i)
                      sf::st_multilinestring (lapply (i,
                             function (j) unname (j))))
-    else if (what == "osm_multipolygons")
+
+    } else if (what == "osm_multipolygons") {
+
         g <- lapply (g, function (i)
                      sf::st_multipolygon (lapply (i, function (j) unname (j))))
+    }
+
     x [[what]]$geometry <- sf::st_sfc (g, crs = 4326)
     return (x)
 }
-
